<?php

declare(strict_types=1);

namespace Drupal\Tests\paragraph_lineage\Unit\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\paragraph_lineage\Form\ParagraphLineageSettingsForm;
use Drupal\paragraph_lineage\ParagraphLineageSettingsServiceInterface;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\MockObject\MockObject;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Tests the ParagraphLineageSettingsForm.
 *
 * @group paragraph_lineage
 * @coversDefaultClass \Drupal\paragraph_lineage\Form\ParagraphLineageSettingsForm
 */
class ParagraphLineageSettingsFormTest extends UnitTestCase {

  /**
   * The configuration factory mock.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface&\PHPUnit\Framework\MockObject\MockObject
   */
  protected ConfigFactoryInterface&MockObject $configFactory;

  /**
   * The typed config manager mock.
   *
   * @var \Drupal\Core\Config\TypedConfigManagerInterface&\PHPUnit\Framework\MockObject\MockObject
   */
  protected TypedConfigManagerInterface&MockObject $typedConfigManager;

  /**
   * The settings service mock.
   *
   * @var \Drupal\paragraph_lineage\ParagraphLineageSettingsServiceInterface&\PHPUnit\Framework\MockObject\MockObject
   */
  protected ParagraphLineageSettingsServiceInterface&MockObject $settingsService;

  /**
   * The form under test.
   *
   * @var \Drupal\paragraph_lineage\Form\ParagraphLineageSettingsForm
   */
  protected ParagraphLineageSettingsForm $form;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    $this->typedConfigManager = $this->createMock(TypedConfigManagerInterface::class);
    $this->settingsService = $this->createMock(ParagraphLineageSettingsServiceInterface::class);

    // Create the actual form instance since it's final.
    $this->form = new ParagraphLineageSettingsForm(
      $this->configFactory,
      $this->typedConfigManager,
      $this->settingsService
    );

    // Mock string translation.
    $string_translation = $this->createMock(TranslationInterface::class);
    $string_translation->method('translate')
      ->willReturnArgument(0);
    $this->form->setStringTranslation($string_translation);
  }

  /**
   * Tests the form ID.
   *
   * @covers ::getFormId
   */
  public function testGetFormId(): void {
    $this->assertEquals('paragraph_lineage_settings', $this->form->getFormId());
  }

  /**
   * Tests getEditableConfigNames.
   *
   * @covers ::getEditableConfigNames
   */
  public function testGetEditableConfigNames(): void {
    $method = new \ReflectionMethod($this->form, 'getEditableConfigNames');
    $method->setAccessible(TRUE);
    $result = $method->invoke($this->form);
    $this->assertEquals(['paragraph_lineage.settings'], $result);
  }

  /**
   * Tests buildForm structure and default values.
   *
   * @covers ::buildForm
   */
  public function testBuildForm(): void {
    $this->settingsService->expects($this->once())
      ->method('getPreferredThemeId')
      ->willReturn('admin');

    $form_state = $this->createMock(FormStateInterface::class);
    $form = [];

    $result = $this->form->buildForm($form, $form_state);

    $this->assertArrayHasKey('preferred_theme', $result);
    $this->assertEquals('radios', $result['preferred_theme']['#type']);
    $this->assertArrayHasKey('#title', $result['preferred_theme']);
    $this->assertArrayHasKey('#description', $result['preferred_theme']);
    $this->assertEquals('admin', $result['preferred_theme']['#default_value']);
    $this->assertArrayHasKey('default', $result['preferred_theme']['#options']);
    $this->assertArrayHasKey('admin', $result['preferred_theme']['#options']);
  }

  /**
   * Tests submitForm calls the settings service.
   *
   * @covers ::submitForm
   */
  public function testSubmitForm(): void {
    // Mock messenger service and container to handle parent::submitForm()
    $messenger = $this->createMock(MessengerInterface::class);
    $messenger->expects($this->once())
      ->method('addStatus');

    $container = $this->createMock(ContainerInterface::class);
    $container->method('get')
      ->with('messenger')
      ->willReturn($messenger);

    \Drupal::setContainer($container);

    $form_state = $this->createMock(FormStateInterface::class);
    $form_state->expects($this->once())
      ->method('getValue')
      ->with('preferred_theme')
      ->willReturn('default');

    $this->settingsService->expects($this->once())
      ->method('setPreferredThemeId')
      ->with('default');

    $form = [];
    $this->form->submitForm($form, $form_state);
  }

}
