<?php

declare(strict_types=1);

namespace Drupal\Tests\paragraph_lineage\Unit;

use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\paragraph_lineage\ParagraphLineageSettingsService;
use Drupal\Tests\UnitTestCase;
use PHPUnit\Framework\MockObject\MockObject;

/**
 * Tests the ParagraphLineageSettingsService.
 *
 * @group paragraph_lineage
 * @coversDefaultClass \Drupal\paragraph_lineage\ParagraphLineageSettingsService
 */
class ParagraphLineageSettingsServiceTest extends UnitTestCase {

  /**
   * The configuration factory mock.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface&\PHPUnit\Framework\MockObject\MockObject
   */
  protected ConfigFactoryInterface&MockObject $configFactory;

  /**
   * The typed config manager mock.
   *
   * @var \Drupal\Core\Config\TypedConfigManagerInterface&\PHPUnit\Framework\MockObject\MockObject
   */
  protected TypedConfigManagerInterface&MockObject $typedConfigManager;

  /**
   * The service under test.
   *
   * @var \Drupal\paragraph_lineage\ParagraphLineageSettingsService
   */
  protected ParagraphLineageSettingsService $settingsService;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->configFactory = $this->createMock(ConfigFactoryInterface::class);
    $this->typedConfigManager = $this->createMock(TypedConfigManagerInterface::class);

    $this->settingsService = new ParagraphLineageSettingsService(
      $this->configFactory,
      $this->typedConfigManager,
      $this->createMock(ThemeManagerInterface::class)
    );
  }

  /**
   * Tests getPreferredThemeId with a configured theme.
   *
   * @covers ::getPreferredThemeId
   */
  public function testGetPreferredThemeIdWithConfiguredTheme(): void {
    $config = $this->createMock(Config::class);
    $config->expects($this->once())
      ->method('get')
      ->with('preferred_theme')
      ->willReturn('custom_theme');

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with(ParagraphLineageSettingsService::CONFIG_NAME)
      ->willReturn($config);

    $result = $this->settingsService->getPreferredThemeId();
    $this->assertEquals('custom_theme', $result);
  }

  /**
   * Tests getPreferredThemeId falls back to admin theme.
   *
   * @covers ::getPreferredThemeId
   */
  public function testGetPreferredThemeIdFallsBackToAdminTheme(): void {
    $settingsConfig = $this->createMock(Config::class);
    $settingsConfig->expects($this->once())
      ->method('get')
      ->with('preferred_theme')
      ->willReturn(NULL);

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with(ParagraphLineageSettingsService::CONFIG_NAME)
      ->willReturn($settingsConfig);

    $result = $this->settingsService->getPreferredThemeId();
    $this->assertEquals('admin', $result);
  }

  /**
   * Tests getPreferredThemeId falls back to admin when no admin theme.
   *
   * @covers ::getPreferredThemeId
   */
  public function testGetPreferredThemeIdFallsBackToAdmin(): void {
    $settingsConfig = $this->createMock(Config::class);
    $settingsConfig->expects($this->once())
      ->method('get')
      ->with('preferred_theme')
      ->willReturn(NULL);

    $this->configFactory->expects($this->once())
      ->method('get')
      ->with(ParagraphLineageSettingsService::CONFIG_NAME)
      ->willReturn($settingsConfig);

    $result = $this->settingsService->getPreferredThemeId();
    $this->assertEquals('admin', $result);
  }

  /**
   * Tests setPreferredThemeId saves the theme.
   *
   * @covers ::setPreferredThemeId
   */
  public function testSetPreferredThemeId(): void {
    $config = $this->createMock(Config::class);
    $config->expects($this->once())
      ->method('set')
      ->with('preferred_theme', 'new_theme')
      ->willReturnSelf();
    $config->expects($this->once())
      ->method('save')
      ->willReturnSelf();

    $this->configFactory->expects($this->once())
      ->method('getEditable')
      ->with(ParagraphLineageSettingsService::CONFIG_NAME)
      ->willReturn($config);

    $this->settingsService->setPreferredThemeId('new_theme');
  }

}
